%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Simplified velocity estimation using discrete Kalman filtering.  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%Summary%%%%%%%




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 1 -PROBLEM DATA DEFINITION-  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %filt_speed : This variable will define the velocity of the estimation. If we increase that variable the filter will arrive soon to a acceptable estimation but the more speed in the filter is going to punish the system with a bigger perturbation.
        %Ts: Defines the sample period time. 
        %accelt:(1x12) vector that represents the real aceleration (for the simulation) each time segment .
        %pos0: Initial position
        %accel0: Initial acceleration
        %w1 : Noise in the position sensor. Gaussian distribution
        %w2 : Noise in the acceleration sensor. Gaussian distribution
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 2 -SIMULATION OF THE REAL POSITION, ACCELERATION AND VELOCITY-   %
%          - GENERATION OF THE ACCELERATION AND POSITION SENSOR SIGNALS-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %accel : Real acceleration.
        %vel : Real velocity.
        %pos : Real position.
        %pos_sen: Position sensor measure vector.
        %accel_sen: Acceleration sensor measure vector
        

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 3 -Space state definition-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %x(k+1)= F(k) x(k) + v(k+1)
        %z(k+1)=H(k+1) x(k+1) + w(k+1)
        %v and w are gaussian noises and their covariance matix are Q and R
        
        %State vector: x=[position, velocity, acceleration]'
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 4 -Kalman filtering-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %We will run kfloop.m
        %The given results will be:
            %pos_est:estimated possition
            %vel_est:estimated velocity
            %accel_est:estimated velocity
        
%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 5 -Results-  %  
%%%%%%%%%%%%%%%%%%%%%%%

    %Results plots

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
clear all;
close all;
format long;


%<<<<<<<<<<<<<STAGE 1>>>>>>>>>>>>>%

        Ts=0.1;                 %We will have a sample frecuency of 0.1kHz
        filt_speed=0.005;        %Filter speed selection

        %Velocity vector definition.Each value represents the values we have
        %created for the example.

        accelt=2*[+0.0  +0.6  -0.1  -1.0  0 -0.0  +0.3  +1.0  0  -1  -0.5 +0.6];

        pos0=0;                  %Initial condition for the position.
        vel0=0;                  %Initial condition for the velocity.
        accel0= accelt(1);       %Initial condition for the acceleration.     

        w1=50;             %Position noise 
        w2=0.5;             %Acceleration noise 
%<<<<<<<<<<<<<END STAGE 1>>>>>>>>>>>>>% 


%<<<<<<<<<<<<<STAGE 2>>>>>>>>>>>>>%

        st=round(10/Ts);    %Number of samples per segment
        Ts=10/st;           %Sampling time after round the number of samples per segment.
        t=Ts:Ts:120;        %Time vector.

        Lt=length(t);                                            % Total number of samples.
        samples=1:1:st*12;                                       % Sample number vector.

        %Object velocity simulation
        accel=[];
                                      
        for i=1:length(accelt);
            accel=[accel;accelt(i)*ones(st,1)];
        end
        
        vel= vel0 + cumsum(accel)*Ts ;     %velocity by the acceleration integration.
        pos= pos0 + cumsum(vel)*Ts;        %position by the velocity integration
        
        pos=pos';
        vel=vel';
        accel=accel';
        
        noise1=w1*randn(1,Lt);          % Gaussian position noise with variance w1^2
        noise2=w2*randn(1,Lt);          % Gaussian acceleration noise with variance w2^2

        pos_sen=pos+noise1;                % Measured position
        accel_sen=accel+noise2;            % Measured acceleration  

 %<<<<<<<<<<<<<END STAGE 2>>>>>>>>>>>>>%  
 
 
 %<<<<<<<<<<<<<STAGE 3>>>>>>>>>>>>>%
 
        A=[0 1 0;0 0 1;0 0 0];
        B=[0 0 0;0 0 0;0 0 0];
        C=[1 0 0;0 0 1];          %Continuous system definition
        D=[0 0 0;0 0 0];
        
        sys=ss(A,B,C,D);
        sys=c2d(sys,Ts,'zoh');  % Discretization 
        [F,G,H,Kd]=ssdata(sys);
        
        %Sensor matrix covariance. The elements that arent in the diagonal
        %are thero because the noise is uncorrelated.
        R=[w1^2   0;0 w2^2];
        
        Q=[ (0.5*Ts^2)*(0.5*Ts^2)   0            0  ;
            0                    Ts*Ts           0  ; 
            0                      0             1*1];
        
        Q=filt_speed*Q/norm(Q);
 
 %<<<<<<<<<<<<<END STAGE 3>>>>>>>>>>>>>%  
 
  %<<<<<<<<<<<<<STAGE 4>>>>>>>>>>>>>%
  
  %Seting the estimated variables starting value
  
    pos_est(1)=pos(1);
    vel_est(1)=0;
    accel_est(1)=accel(1);
  
    P(:,:,1)= Q;
    XKK= [pos_est(1); vel_est(1);  accel_est(1)];    %X(k|k) (estimated)
    PKK=P(:,:,1);                                   %P(k|k) initial value
    
    % Calling the Kallman  loop
    
        for  k=1:Lt-1

            ZK1=[pos(k+1); accel(k+1)];             %Z(k+1) measured at k+1
            %XK1K1=X(k+1|k+1) PK1K1=P(k+1|k+1) state covariance
            [XK1K1,PK1K1,KK1]=fkloop(XKK,PKK,ZK1,F,H,Q,R); 
            
            
            
             Ka(:,:,k+1)=KK1;                       %Ka represents the evolution of the kalman gain    
             pos_est(k+1)=XK1K1(1);                 %Estimated position given for the Kalman filter for the next step
             vel_est(k+1)=XK1K1(2);                 %Estimated velocity given for the Kalman filter for the next step     
             accel_est(k+1)=XK1K1(3);               %Estimated acceleration given for the Kalman filter for the next step
             P(:,:,k+1)=PK1K1;
        
        %t+1 will be t in the next step in the loop
              PKK=PK1K1;
              XKK=XK1K1;
        
        end
 %<<<<<<<<<<<<<END STAGE 4>>>>>>>>>>>>>%
 
 
 %<<<<<<<<<<<<<STAGE 5>>>>>>>>>>>>>%
 
 %%%Measurements plots%%%
        figure('Name','Sensor messurements')
        subplot(2,1,1)
        plot(t,pos,t,pos_sen)
        ylabel('position (m)')
        xlabel('time (s)')
        legend('Real position','Position sensor measurement')
        title(strcat('sensor n1, sigma(noise)=',num2str(std(noise1)),' m')) 
        grid on


        subplot(2,1,2)
        plot(t,accel,t,accel_sen)
        ylabel('acceleration (m/s^2)')
        xlabel('time (s)')
        legend('Real acceleration','Acceleration sensor measurement')
        title(strcat('sensor n2, sigma(noise)=',num2str(std(noise2)),' m/s^2')) 
        grid on

    %%%Kalman gain evolution%%%
    
        figure('Name','Kallman gain evolution')
        subplot(3,2,1)
        K11(:,1,1)=Ka(1,1,:);
        plot(t,K11)
        title('K(1,1)')
        xlabel('time (s)')
        grid on
        
        
        subplot(3,2,2)
        K12(:,1,1)=Ka(1,2,:);
        plot(t,K12)
        title('K(1,2)')
        xlabel('time (s)')
        grid on
        
        
        subplot(3,2,3)
        K21(:,1,1)=Ka(2,1,:);
        plot(t,K21)
        title('K(2,1)')
        xlabel('time (s)')
        grid on
        
        subplot(3,2,4)
        K22(:,1,1)=Ka(2,2,:);
        plot(t,K22)
        title('K(2,2)')
        xlabel('time (s)')
        grid on
        
        subplot(3,2,4)
        K22(:,1,1)=Ka(2,2,:);
        plot(t,K22)
        title('K(2,2)')
        xlabel('time (s)')
        grid on
        
        subplot(3,2,5)
        K31(:,1,1)=Ka(3,1,:);
        plot(t,K31)
        title('K(3,1)')
        xlabel('time (s)')
        grid on
        
        subplot(3,2,6)
        K32(:,1,1)=Ka(3,2,:);
        plot(t,K32)
        title('K(3,2)')
        xlabel('time (s)')
        grid on
        
        
        
 %%%P evolution%%%%
 
 figure('Name','P evolution')


        subplot(3,3,1)
        P11(:,1,1)=P(1,1,:);
        plot(t,P11)
        title('P(1,1)')
        xlabel('time (s)')
        grid on

        subplot(3,3,2)
        P12(:,1,1)=P(1,2,:);
        plot(t,P12)
        title('P(1,2)')
        xlabel('time (s)')
        grid on

        subplot(3,3,3)
        P13(:,1,1)=P(1,3,:);
        plot(t,P13)
        title('P(1,3)')
        xlabel('time (s)')
        grid on

        subplot(3,3,4)
        P21(:,1,1)=P(2,1,:);
        plot(t,P21)
        title('P(2,1)')
        xlabel('time (s)')
        grid on

        subplot(3,3,5)
        P22(:,1,1)=P(2,2,:);
        plot(t,P22)
        title('P(2,2)')
        xlabel('time (s)')
        grid on

        subplot(3,3,6)
        P23(:,1,1)=P(2,3,:);
        plot(t,P23)
        title('P(2,3)')
        xlabel('time (s)')
        grid on

        subplot(3,3,7)
        P31(:,1,1)=P(3,1,:);
        plot(t,P31)
        title('P(3,1)')
        xlabel('time (s)')
        grid on

        subplot(3,3,8)
        P32(:,1,1)=P(3,2,:);
        plot(t,P32)
        title('P(3,2)')
        xlabel('time (s)')
        grid on

        subplot(3,3,9)
        P33(:,1,1)=P(3,3,:);
        plot(t,P33)
        title('P(3,3)')
        xlabel('time (s)')
        grid on
        
%%%Estimated measurement vs real ones%%%

        figure('Name','Real state vd estimated state')

        subplot(3,1,1)
        plot(t,pos,t,pos_est)
        ylabel('position (m)')
        xlabel('time (s)')
        legend('Real position','Estimated position')
        title(strcat('Ts=',num2str(Ts),  '  sigma(sensor n1)=',num2str(std(noise1)), 'sigma(sensor n2)=',num2str(std(noise2))))
        grid on

        subplot(3,1,2)
        plot(t,vel,t,vel_est)
        ylabel('Velocity (m/s)')
        xlabel('time (s)')
        legend('Real velocity','Estimated velocity')
        grid on

        subplot(3,1,3)
        plot(t,accel,t,accel_est)
        ylabel('Acceleration(m/s^2)')
        xlabel('time (s)')
        legend('Real acceleration','Estimated acceleration')
        grid on